"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateEmit = exports.emitEvent = void 0;
var tslib_1 = require("tslib");
var common_1 = require("../common");
var lodash_es_1 = require("lodash-es");
var global_api_1 = require("../../app/global-api");
var adapter_1 = require("../adapter");
var mobx_1 = require("mobx");
var widgets_1 = require("../widgets");
var DEFAULT_MAX_TIMEOUT = 10 * 1000;
function emitEvent(trigger, listeners, args, scopeContext, dataContext, $w, pageId) {
    if (listeners === void 0) { listeners = []; }
    if (scopeContext === void 0) { scopeContext = {}; }
    if (pageId === void 0) { pageId = ''; }
    var targetListeners = listeners.filter(function (l) { return l.trigger === trigger; });
    function processListener(listener) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var event, res, e_1, res, e_2;
            return tslib_1.__generator(this, function (_k) {
                switch (_k.label) {
                    case 0:
                        // 当前非捕获Event，再判断冒泡行为
                        if (!((_a = args === null || args === void 0 ? void 0 : args.event) === null || _a === void 0 ? void 0 : _a.isCapturePhase) && listener.noPropagation) {
                            ((_b = args === null || args === void 0 ? void 0 : args.domEvent) === null || _b === void 0 ? void 0 : _b.stopPropagation) && ((_c = args === null || args === void 0 ? void 0 : args.domEvent) === null || _c === void 0 ? void 0 : _c.stopPropagation());
                        }
                        event = {
                            detail: args.event.detail,
                            isCapturePhase: !!((_d = args.event) === null || _d === void 0 ? void 0 : _d.isCapturePhase),
                            origin: args.event,
                            name: '',
                        };
                        if (!(((listener === null || listener === void 0 ? void 0 : listener.isCapturePhase) || false) === (((_e = args === null || args === void 0 ? void 0 : args.customEventData) === null || _e === void 0 ? void 0 : _e.isCapturePhase) || false))) return [3 /*break*/, 8];
                        if (pageId && ((_f = $w === null || $w === void 0 ? void 0 : $w.page) === null || _f === void 0 ? void 0 : _f.id) && pageId !== ((_g = $w === null || $w === void 0 ? void 0 : $w.page) === null || _g === void 0 ? void 0 : _g.id)) {
                            console.error("Action error: [".concat(listener.trigger, "] \u9875\u9762\u751F\u547D\u5468\u671F\u7ED3\u675F\uFF0C\u94FE\u5F0F\u8C03\u7528\u4E2D\u65AD"));
                            return [2 /*return*/];
                        }
                        if (!((listener === null || listener === void 0 ? void 0 : listener.sourceKey) === 'platform:utils.If')) return [3 /*break*/, 5];
                        _k.label = 1;
                    case 1:
                        _k.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, invokeListener(listener, args, scopeContext, dataContext, $w)];
                    case 2:
                        res = _k.sent();
                        event.name = "".concat(listener.key, ".").concat(res ? 'success' : 'fail');
                        return [3 /*break*/, 4];
                    case 3:
                        e_1 = _k.sent();
                        event.name = "".concat(listener.key, ".fail");
                        return [3 /*break*/, 4];
                    case 4: return [3 /*break*/, 8];
                    case 5:
                        _k.trys.push([5, 7, , 8]);
                        return [4 /*yield*/, invokeListener(listener, args, scopeContext, dataContext, $w)];
                    case 6:
                        res = _k.sent();
                        event.name = "".concat(listener.key, ".success");
                        event.detail = res;
                        return [3 /*break*/, 8];
                    case 7:
                        e_2 = _k.sent();
                        if (e_2.message) {
                            e_2.message = "".concat(((_j = (_h = args === null || args === void 0 ? void 0 : args.event) === null || _h === void 0 ? void 0 : _h.currentTarget) === null || _j === void 0 ? void 0 : _j.id) || '', "[").concat(listener.trigger, "] ").concat(e_2.message);
                        }
                        event.name = "".concat(listener.key, ".fail");
                        event.detail = e_2;
                        // 之前 invoke 内部catch 了错误，不会抛错
                        // throw e
                        console.error("Action error: [".concat(listener.trigger, "]"), e_2);
                        return [3 /*break*/, 8];
                    case 8:
                        emitEvent(event.name, listeners, tslib_1.__assign(tslib_1.__assign({}, args), { event: event, customEventData: event }), scopeContext, dataContext, $w, pageId);
                        return [2 /*return*/];
                }
            });
        });
    }
    if (!targetListeners.length && typeof trigger === 'string' && /\.fail$/.test(trigger)) {
        global_api_1.app.showToast({
            title: "\u4E8B\u4EF6\u54CD\u5E94\u5931\u8D25\uFF1A".concat(args.event.detail),
            icon: 'error',
        });
    }
    else {
        targetListeners.forEach(function (listener) {
            processListener(listener);
        });
    }
}
exports.emitEvent = emitEvent;
function generateEmit($w, currentWidget, // page 节点暂未生成
injectContext, extraListeners, extraForContext) {
    if (currentWidget === void 0) { currentWidget = undefined; }
    if (injectContext === void 0) { injectContext = undefined; }
    if (extraListeners === void 0) { extraListeners = []; }
    if (extraForContext === void 0) { extraForContext = {}; }
    return function (trigger, listeners, eventData, forItems, domEvent, scopeContext, dataContext, pageId) {
        if (listeners === void 0) { listeners = []; }
        if (pageId === void 0) { pageId = ''; }
        listeners = listeners.concat(extraListeners);
        if (injectContext === null || injectContext === void 0 ? void 0 : injectContext.emit) {
            return injectContext === null || injectContext === void 0 ? void 0 : injectContext.emit(trigger, listeners, eventData, forItems, domEvent, scopeContext, dataContext);
        }
        else {
            var event_1 = {
                detail: eventData,
                name: trigger,
                target: currentWidget,
                currentTarget: currentWidget,
                domEvent: domEvent,
            };
            var forContext_1 = tslib_1.__assign(tslib_1.__assign({}, forItems), extraForContext);
            emitEvent(trigger, listeners, {
                event: event_1,
                customEventData: event_1,
                forItems: forContext_1,
                domEvent: domEvent,
            }, scopeContext, dataContext, (0, mobx_1.untracked)(function () {
                return (0, widgets_1.generateWidgetAPIContext)($w, currentWidget, forContext_1, scopeContext);
            }), pageId);
        }
    };
}
exports.generateEmit = generateEmit;
function invokeListener(listener, args, scopeContext, dataContext, $w) {
    var _a, _b, _c;
    var key = listener.key, instanceFunction = listener.instanceFunction, _d = listener.data, data = _d === void 0 ? {} : _d, _e = listener.dataBinds, dataBinds = _e === void 0 ? {} : _e;
    if (!instanceFunction) {
        try {
            // @ts-ignore
            (_b = (_a = window._aegis) === null || _a === void 0 ? void 0 : _a.error) === null || _b === void 0 ? void 0 : _b.call(_a, 'SYSTEM_ERROR: listener not found', listener);
            console.error('$page current hander', (_c = adapter_1.runtime === null || adapter_1.runtime === void 0 ? void 0 : adapter_1.runtime['$page']) === null || _c === void 0 ? void 0 : _c.handler);
        }
        catch (e) { }
        throw new Error("".concat(key, " handler \u672A\u627E\u5230'"));
    }
    // ToDo resolve databinds
    var action = instanceFunction;
    var resolvedData = tslib_1.__assign({}, data);
    var resolvedDataBinds = (0, common_1.resolveDataBinds)(dataBinds, args.forItems, { event: args.event }, scopeContext, true, dataContext, $w);
    // eslint-disable-next-line no-restricted-syntax
    for (var key_1 in resolvedDataBinds) {
        if (resolvedDataBinds[key_1] && resolvedDataBinds[key_1].__type === 'scopedValue') {
            try {
                (0, lodash_es_1.set)(resolvedData, key_1, resolvedDataBinds[key_1].getValue(scopeContext));
            }
            catch (e) {
                (0, lodash_es_1.set)(resolvedData, key_1, '');
            }
        }
        else {
            (0, lodash_es_1.set)(resolvedData, key_1, resolvedDataBinds[key_1]);
        }
    }
    var params = tslib_1.__assign({ data: resolvedData }, args);
    return action(params);
}
//# sourceMappingURL=actionHandler.js.map